@@ -1,5 +1,5 @@
 /*--------------------------------------------------------------------
- *	$Id: grdinfo.c 17560 2017-02-17 22:05:42Z pwessel $
+ *	$Id: grdinfo.c 18172 2017-05-07 02:44:25Z pwessel $
  *
  *	Copyright (c) 1991-2017 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
  *	See LICENSE.TXT file for copying and redistribution conditions.
@@ -25,20 +25,21 @@
  *
  */
 
+#include "gmt_dev.h"
+
 #define THIS_MODULE_NAME	"grdinfo"
 #define THIS_MODULE_LIB		"core"
 #define THIS_MODULE_PURPOSE	"Extract information from grids"
 #define THIS_MODULE_KEYS	"<G{+,>T},>DC"
-
-#include "gmt_dev.h"
-
-#define GMT_PROG_OPTIONS "->RVfh"
+#define THIS_MODULE_NEEDS	""
+#define THIS_MODULE_OPTIONS "->RVfh"
 
 /* Control structure for grdinfo */
 
 enum Opt_I_modes {
 	GRDINFO_GIVE_INCREMENTS = 0,
 	GRDINFO_GIVE_REG_ORIG,
+	GRDINFO_GIVE_REG_IMG,
 	GRDINFO_GIVE_REG_ROUNDED,
 	GRDINFO_GIVE_BOUNDBOX};
 
@@ -46,10 +47,15 @@ struct GRDINFO_CTRL {
 	struct GRDINFO_C {	/* -C */
 		bool active;
 	} C;
+	struct GRDINFO_D {	/* -D[dx[/dy]][+i] */
+		bool active;
+		unsigned int mode;
+		double inc[2];
+	} D;
 	struct GRDINFO_F {	/* -F */
 		bool active;
 	} F;
-	struct GRDINFO_I {	/* -Idx[/dy] */
+	struct GRDINFO_I {	/* -Ir|b|i|dx[/dy] */
 		bool active;
 		unsigned int status;
 		double inc[2];
@@ -57,7 +63,7 @@ struct GRDINFO_CTRL {
 	struct GRDINFO_M {	/* -M */
 		bool active;
 	} M;
-	struct GRDINFO_L {	/* -L[1|2] */
+	struct GRDINFO_L {	/* -L[0|1|2|p] */
 		bool active;
 		unsigned int norm;
 	} L;
@@ -91,7 +97,7 @@ GMT_LOCAL void Free_Ctrl (struct GMT_CTRL *GMT, struct GRDINFO_CTRL *C) {	/* Dea
 GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
 	gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_PURPOSE);
 	if (level == GMT_MODULE_PURPOSE) return (GMT_NOERROR);
-	GMT_Message (API, GMT_TIME_NONE, "usage: grdinfo <grid> [-C] [-F] [-I[<dx>[/<dy>]|r|b]] [-L[0|1|2]] [-M]\n");
+	GMT_Message (API, GMT_TIME_NONE, "usage: grdinfo <grid> [-C] [-D[<offx>[/<offy>][+i]] [-F] [-I[<dx>[/<dy>]|b|i|r]] [-L[a|0|1|2|p]] [-M]\n");
 	GMT_Message (API, GMT_TIME_NONE, "	[%s] [-T[<dz>][+a[<alpha>]][+s]] [%s] [%s]\n\t[%s]\n\n", GMT_Rgeo_OPT, GMT_V_OPT, GMT_f_OPT, GMT_ho_OPT);
 
 	if (level == GMT_SYNOPSIS) return (GMT_MODULE_SYNOPSIS);
@@ -99,18 +105,27 @@ GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
 	GMT_Message (API, GMT_TIME_NONE, "\t<grid> may be one or more grid files.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\n\tOPTIONS:\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-C Format report in fields on a single line using the format\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   file w e s n z0 z1 dx dy n_columns n_rows [x0 y0 x1 y1] [med scale] [mean std rms] [n_nan].\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   (-M gives [x0 y0 x1 y1] and [n_nan]; -L1 gives [med scale]; -L2 gives [mean std rms]).\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   file w e s n z0 z1 dx dy n_columns n_rows [x0 y0 x1 y1] [med L1scale] [mean std rms] [n_nan] [mode LMSscale].\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   (-M gives [x0 y0 x1 y1] and [n_nan], -L1 gives [median L1scale], -L2 gives [mean std rms],\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   while -Lp gives [mode LMSscale]).\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t-D Report tiles using tile size set in -I. Optionally, extend each tile region by <offx>/<offy>.\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   Append +i to only report tiles if the subregion has data (limited to one input grid).\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   If no grid is given then -R must be given and we tile based on the given region.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-F Report domain in world mapping format [Default is generic].\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-I Return textstring -Rw/e/s/n to nearest multiple of dx/dy.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   If -C is set then rounding off will occur but no -R string is issued.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   If no argument is given then the -I<xinc>/<yinc> string is issued.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   If -Ir is given then the grid's -R string is issued.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   If -Ib is given then the grid's bounding box polygon is issued.\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   If -Ii is given then the original img2grd -R string is issued, if available.\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t     If the grid is not an img grid then the regular -R string is issued.\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   If -Ir is given then the grid's -R string is issued.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-L Set report mode:\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   -L0 reports range of data by actually reading them (not from header).\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   -L1 reports median and L1-scale of data set.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   -L[2] reports mean, standard deviation, and rms of data set.\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   -Lp reports mode (lms) and LMS-scale of data set.\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   -La all of the above.\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   If grid is geographic then we report area-weighted statistics.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-M Search for the global min and max locations (x0,y0) and (x1,y1).\n");
 	GMT_Option (API, "R");
 	GMT_Message (API, GMT_TIME_NONE, "\t-T Print global -Tzmin/zmax[/dz] (in rounded multiples of dz, if given).\n");
@@ -130,8 +145,9 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct GRDINFO_CTRL *Ctrl, struct GMT
 	 * returned when registering these sources/destinations with the API.
 	 */
 
+	bool no_file_OK;
 	unsigned int n_errors = 0, n_files = 0;
-char text[GMT_LEN32] = {""};
+	char text[GMT_LEN32] = {""}, *c = NULL;
 	struct GMT_OPTION *opt = NULL;
 
 	for (opt = options; opt; opt = opt->next) {	/* Process all the options given */
@@ -151,6 +167,19 @@ char text[GMT_LEN32] = {""};
 			case 'C':	/* Column format */
 				Ctrl->C.active = true;
 				break;
+			case 'D':	/* Tiling output w/ optional overlap */
+				Ctrl->D.active = true;
+				if (opt->arg[0]) {
+					if ((c = strstr (opt->arg, "+i"))) {
+						c[0] = '\0';	/* Temporarily chop off modifier */
+						Ctrl->D.mode = 1;
+					}
+					if (opt->arg[0] && gmt_getinc (GMT, opt->arg, Ctrl->D.inc)) {
+						gmt_inc_syntax (GMT, 'D', 1);
+						n_errors++;
+					}
+				}
+				break;
 			case 'F':	/* World mapping format */
 				Ctrl->F.active = true;
 				break;
@@ -162,10 +191,12 @@ char text[GMT_LEN32] = {""};
 				Ctrl->I.active = true;
 				if (!opt->arg[0])	/* No args given, we want to output the -I string */
 					Ctrl->I.status = GRDINFO_GIVE_INCREMENTS;
-				else if ((opt->arg[0] == 'r' || opt->arg[0] == '-') && opt->arg[1] == '\0')	/* -Ir: we want to output the actual -R string */
-					Ctrl->I.status = GRDINFO_GIVE_REG_ORIG;
 				else if (opt->arg[0] == 'b' && opt->arg[1] == '\0')	/* -Ib means return grid perimeter as bounding box */
 					Ctrl->I.status = GRDINFO_GIVE_BOUNDBOX;
+				else if (opt->arg[0] == 'i' && opt->arg[1] == '\0')	/* -Ii means return -R string from original img2grd */
+					Ctrl->I.status = GRDINFO_GIVE_REG_IMG;
+				else if ((opt->arg[0] == 'r' || opt->arg[0] == '-') && opt->arg[1] == '\0')	/* -Ir: we want to output the actual -R string */
+					Ctrl->I.status = GRDINFO_GIVE_REG_ORIG;
 				else {	/* Report -R to nearest given multiple increment */
 					Ctrl->I.status = GRDINFO_GIVE_REG_ROUNDED;
 					if (gmt_getinc (GMT, opt->arg, Ctrl->I.inc)) {
@@ -181,6 +212,10 @@ char text[GMT_LEN32] = {""};
 						Ctrl->L.norm |= 2; break;
 					case '1':
 						Ctrl->L.norm |= 1; break;
+					case 'p':
+						Ctrl->L.norm |= 4; break;
+					case 'a':	/* All three */
+						Ctrl->L.norm |= (1+2+4); break;
 				}
 				break;
 			case 'M':	/* Global extrema */
@@ -217,7 +252,9 @@ char text[GMT_LEN32] = {""};
 		}
 	}
 
-	n_errors += gmt_M_check_condition (GMT, n_files == 0, "Syntax error: Must specify one or more input files\n");
+	no_file_OK = (Ctrl->D.active && Ctrl->D.mode == 0 && GMT->common.R.active[RSET]);
+	n_errors += gmt_M_check_condition (GMT, n_files == 0 && !no_file_OK, "Syntax error: Must specify one or more input files\n");
+	n_errors += gmt_M_check_condition (GMT, Ctrl->D.mode && n_files != 1, "Syntax error -D: The +n modifier requires a single grid file\n");
 	n_errors += gmt_M_check_condition (GMT, Ctrl->T.active && Ctrl->T.inc < 0.0, "Syntax error -T: The optional increment must be positive\n");
 	n_errors += gmt_M_check_condition (GMT, Ctrl->T.mode & 2 && n_files != 1, "Syntax error -T: The optional alpha-trim value can only work with a single grid file\n");
 	n_errors += gmt_M_check_condition (GMT, Ctrl->T.active && (Ctrl->T.alpha < 0.0 || Ctrl->T.alpha > 100.0), "Syntax error -T: The optional alpha-trim value must be in the 0 < alpha < 100 %% range\n");
@@ -229,6 +266,85 @@ char text[GMT_LEN32] = {""};
 	return (n_errors ? GMT_PARSE_ERROR : GMT_NOERROR);
 }
 
+struct GMT_TILES {
+	double wesn[4];
+};
+
+GMT_LOCAL void report_tiles (struct GMT_CTRL *GMT, struct GMT_GRID *G, double w, double e, double s, double n, struct GRDINFO_CTRL *Ctrl) {
+	/* Find the tiles covering the present grid, if given */
+	bool use = true;
+	unsigned int nx, ny, i, j, js = 0, jn = 0, ie, iw;
+	uint64_t row, col, node;
+	double wesn[4], out[4], box[4];
+	char text[GMT_LEN64] = {""}, record[GMT_BUFSIZ] = {""};
+	char *sep = GMT->current.setting.io_col_separator;
+	
+	nx = gmt_M_get_n (GMT, w, e, Ctrl->I.inc[GMT_X], 1);
+	ny = gmt_M_get_n (GMT, s, n, Ctrl->I.inc[GMT_Y], 1);
+	for (j = 0; j < ny; j++) {
+		box[YLO] = wesn[YLO] = s + j * Ctrl->I.inc[GMT_Y];	box[YHI] = wesn[YHI] = wesn[YLO] + Ctrl->I.inc[GMT_Y];
+		if (G && Ctrl->D.mode) {	/* Must determine if there is at least one data point inside this subset */
+			if (wesn[YLO] < G->header->wesn[YLO]) wesn[YLO] = G->header->wesn[YLO];
+			if (wesn[YHI] > G->header->wesn[YHI]) wesn[YHI] = G->header->wesn[YHI];
+			js = gmt_M_grd_y_to_row (GMT, wesn[YLO], G->header);
+			jn = gmt_M_grd_y_to_row (GMT, wesn[YHI], G->header);
+		}
+		for (i = 0; i < nx; i++) {
+			box[XLO] = wesn[XLO] = w + i * Ctrl->I.inc[GMT_X];	box[XHI] = wesn[XHI] = wesn[XLO] + Ctrl->I.inc[GMT_X];
+			if (G && Ctrl->D.mode) {	/* Must determine if there is at least one data point inside this subset */
+				if (wesn[XLO] < G->header->wesn[XLO]) wesn[XLO] = G->header->wesn[XLO];
+				if (wesn[XHI] > G->header->wesn[XHI]) wesn[XHI] = G->header->wesn[XHI];
+				iw = gmt_M_grd_x_to_col (GMT, wesn[XLO], G->header);
+				ie = gmt_M_grd_x_to_col (GMT, wesn[XHI], G->header);
+				use = true;
+				for (row = jn; row <= js; row++) {
+					for (col = iw; col <= ie; col++) {
+						node = gmt_M_ijp (G->header, row, col);
+						if (!gmt_M_is_fnan (G->data[node]))
+							goto L_use_it;
+					}
+				}
+				use = false;	/* Could not find a single valid node */
+L_use_it:		row = 0;	/* Get here by goto and use is still true */	
+			}
+			if (use) {
+				gmt_M_memcpy (out, box, 4, double);
+				out[XLO] -= Ctrl->D.inc[GMT_X];
+				out[XHI] += Ctrl->D.inc[GMT_X];
+				out[YLO] -= Ctrl->D.inc[GMT_Y];
+				out[YHI] += Ctrl->D.inc[GMT_Y];
+				if (gmt_M_is_geographic (GMT, GMT_IN)) {	/* Must make sure we don't get outside valid bounds */
+					if (out[YLO] < -90.0)
+						out[YLO] = -90.0;
+					if (out[YHI] > 90.0)
+						out[YHI] = 90.0;
+					if (fabs (out[XHI] - out[XLO]) > 360.0) {
+						out[XLO] = (out[XLO] < 0.0) ? -180.0 : 0.0;
+						out[XHI] = (out[XHI] < 0.0) ? +180.0 : 360.0;
+					}
+				}
+				if (GMT->parent->external)	/* External interface */
+					GMT_Put_Record (GMT->parent, GMT_WRITE_DOUBLE, out);
+				else if (Ctrl->C.active) {
+					gmt_ascii_format_col (GMT, record, out[XLO], GMT_OUT, GMT_X);	strcat (record, sep);
+					gmt_ascii_format_col (GMT, text, out[XHI], GMT_OUT, GMT_X);	strcat (record, text);	strcat (record, sep);
+					gmt_ascii_format_col (GMT, text, out[YLO], GMT_OUT, GMT_Y);	strcat (record, text);	strcat (record, sep);
+					gmt_ascii_format_col (GMT, text, out[YHI], GMT_OUT, GMT_Y);	strcat (record, text);
+					GMT_Put_Record (GMT->parent, GMT_WRITE_TEXT, record);
+				}
+				else {
+					sprintf (record, "-R");
+					gmt_ascii_format_col (GMT, text, out[XLO], GMT_OUT, GMT_X);	strcat (record, text);	strcat (record, "/");
+					gmt_ascii_format_col (GMT, text, out[XHI], GMT_OUT, GMT_X);	strcat (record, text);	strcat (record, "/");
+					gmt_ascii_format_col (GMT, text, out[YLO], GMT_OUT, GMT_Y);	strcat (record, text);	strcat (record, "/");
+					gmt_ascii_format_col (GMT, text, out[YHI], GMT_OUT, GMT_X);	strcat (record, text);
+					GMT_Put_Record (GMT->parent, GMT_WRITE_TEXT, record);
+				}
+			}
+		}
+	}
+}
+
 #define bailout(code) {gmt_M_free_options (mode); return (code);}
 #define Return(code) {Free_Ctrl (GMT, Ctrl); gmt_end_module (GMT, GMT_cpy); bailout (code);}
 
@@ -239,20 +355,20 @@ char text[GMT_LEN32] = {""};
 
 int GMT_grdinfo (void *V_API, int mode, void *args) {
 	int error = 0;
-	unsigned int n_grds = 0, o_type = GMT_IS_TEXTSET, n_cols = 0, col;
-	bool subset;
+	unsigned int n_grds = 0, o_type = GMT_IS_TEXTSET, n_cols = 0, col, i_status;
+	bool subset, delay;
 
 	uint64_t ij, n_nan = 0, n = 0;
 
 	double x_min = 0.0, y_min = 0.0, z_min = 0.0, x_max = 0.0, y_max = 0.0, z_max = 0.0, wesn[4];
 	double global_xmin, global_xmax, global_ymin, global_ymax, global_zmin, global_zmax;
-	double mean = 0.0, median = 0.0, sum2 = 0.0, stdev = 0.0, scale = 0.0, rms = 0.0, x, out[20];
+	double z_mean = 0.0, z_median = 0.0, z_mode = 0.0, z_stdev = 0.0, z_scale = 0.0, z_lmsscl = 0.0, z_rms = 0.0, out[22];
 
 	char format[GMT_BUFSIZ] = {""}, text[GMT_LEN64] = {""}, record[GMT_BUFSIZ] = {""}, grdfile[GMT_LEN256] = {""};
 	char *type[2] = { "Gridline", "Pixel"}, *sep = NULL, *projStr = NULL;
 
 	struct GRDINFO_CTRL *Ctrl = NULL;
-	struct GMT_GRID *G = NULL;
+	struct GMT_GRID *G = NULL, *W = NULL;
 	struct GMT_OPTION *opt = NULL;
 	struct GMT_CTRL *GMT = NULL, *GMT_cpy = NULL;
 	struct GMT_OPTION *options = NULL;
@@ -269,8 +385,8 @@ int GMT_grdinfo (void *V_API, int mode, void *args) {
 
 	/* Parse the command-line arguments */
 
-	GMT = gmt_begin_module (API, THIS_MODULE_LIB, THIS_MODULE_NAME, &GMT_cpy); /* Save current state */
-	if (GMT_Parse_Common (API, GMT_PROG_OPTIONS, options)) Return (API->error);
+	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
+	if (GMT_Parse_Common (API, THIS_MODULE_OPTIONS, options)) Return (API->error);
 	Ctrl = New_Ctrl (GMT);	/* Allocate and initialize a new control structure */
 	if ((error = parse (GMT, Ctrl, options)) != 0) Return (error);
 
@@ -280,16 +396,26 @@ int GMT_grdinfo (void *V_API, int mode, void *args) {
 	
 	sep = GMT->current.setting.io_col_separator;
 	gmt_M_memcpy (wesn, GMT->common.R.wesn, 4, double);	/* Current -R setting, if any */
-	global_xmin = global_ymin = global_zmin = DBL_MAX;
-	global_xmax = global_ymax = global_zmax = -DBL_MAX;
+	if (Ctrl->D.active && Ctrl->D.mode == 0 && GMT->common.R.active[RSET]) {
+		global_xmin = GMT->common.R.wesn[XLO]; global_ymin = GMT->common.R.wesn[YLO];
+		global_xmax = GMT->common.R.wesn[XHI] ; global_ymax = GMT->common.R.wesn[YHI];
+		global_zmin = DBL_MAX;		global_zmax = -DBL_MAX;
+	}
+	else {
+		global_xmin = global_ymin = global_zmin = DBL_MAX;
+		global_xmax = global_ymax = global_zmax = -DBL_MAX;
+	}
+	delay = (Ctrl->D.mode == 1 || (Ctrl->T.mode & 2));	/* Delay the freeing of the (single) grid we read */
+	
 	if (Ctrl->C.active) {
-		if (API->mode) o_type = GMT_IS_DATASET;	/* With external interface we are returning doubles */
+		if (API->external) o_type = GMT_IS_DATASET;	/* With external interface we are returning doubles */
 		n_cols = 6;	/* w e s n z0 z1 */
 		if (!Ctrl->I.active) {
 			n_cols += 4;				/* Add dx dy n_columns n_rows */
 			if (Ctrl->M.active) n_cols += 5;	/* Add x0 y0 x1 y1 nnan */
 			if (Ctrl->L.norm & 1) n_cols += 2;	/* Add median scale */
 			if (Ctrl->L.norm & 2) n_cols += 3;	/* Add mean stdev rms */
+			if (Ctrl->L.norm & 4) n_cols += 2;	/* Add mode lmsscale */
 		}
 	}
 	if (GMT_Init_IO (API, o_type, GMT_IS_NONE, GMT_OUT, GMT_ADD_DEFAULT, 0, options) != GMT_NOERROR) {	/* Registers default output destination, unless already set */
@@ -314,7 +440,7 @@ int GMT_grdinfo (void *V_API, int mode, void *args) {
 
 		gmt_set_cartesian (GMT, GMT_IN);	/* Reset since we may get a bunch of files, some geo, some not */
 
-		if ((G = GMT_Read_Data (API, GMT_IS_GRID, GMT_IS_FILE, GMT_IS_SURFACE, GMT_GRID_HEADER_ONLY, NULL, opt->arg, NULL)) == NULL) {
+		if ((G = GMT_Read_Data (API, GMT_IS_GRID, GMT_IS_FILE, GMT_IS_SURFACE, GMT_CONTAINER_ONLY, NULL, opt->arg, NULL)) == NULL) {
 			Return (API->error);
 		}
 		subset = gmt_M_is_subset (GMT, G->header, wesn);	/* Subset requested */
@@ -331,8 +457,8 @@ int GMT_grdinfo (void *V_API, int mode, void *args) {
 
 		n_grds++;
 
-		if (Ctrl->M.active || Ctrl->L.active || subset || (Ctrl->T.mode & 2)) {	/* Need to read the data (all or subset) */
-			if (GMT_Read_Data (API, GMT_IS_GRID, GMT_IS_FILE, GMT_IS_SURFACE, GMT_GRID_DATA_ONLY, wesn, opt->arg, G) == NULL) {
+		if (Ctrl->M.active || Ctrl->L.active || subset || Ctrl->D.mode || (Ctrl->T.mode & 2)) {	/* Need to read the data (all or subset) */
+			if (GMT_Read_Data (API, GMT_IS_GRID, GMT_IS_FILE, GMT_IS_SURFACE, GMT_DATA_ONLY, wesn, opt->arg, G) == NULL) {
 				Return (API->error);
 			}
 		}
@@ -344,7 +470,6 @@ int GMT_grdinfo (void *V_API, int mode, void *args) {
 			unsigned int col, row;
 
 			z_min = DBL_MAX;	z_max = -DBL_MAX;
-			mean = median = sum2 = 0.0;
 			ij_min = ij_max = n = 0;
 			gmt_M_grd_loop (GMT, G, row, col, ij) {
 				if (gmt_M_is_fnan (G->data[ij])) continue;
@@ -355,11 +480,6 @@ int GMT_grdinfo (void *V_API, int mode, void *args) {
 					z_max = G->data[ij];	ij_max = ij;
 				}
 				n++;
-				if (Ctrl->L.active) {	/* Use Welford (1962) algorithm to compute mean and corrected sum of squares */
-					x = G->data[ij] - mean;
-					mean += x / n;
-					sum2 += x * (G->data[ij] - mean);
-				}
 			}
 
 			n_nan = G->header->nm - n;
@@ -377,31 +497,25 @@ int GMT_grdinfo (void *V_API, int mode, void *args) {
 				x_min = x_max = y_min = y_max = GMT->session.d_NaN;
 		}
 
-		if (Ctrl->L.norm & 1) {	/* Calculate the median and L1 scale */
-			int new_grid;
-			struct GMT_GRID *G2 = NULL;
-
-			/* Note that this option rearranges the input grid, so if a memory location is passed then
-			 * the grid in the calling program is no longer the original values */
-			new_grid = gmt_set_outgrid (GMT, opt->arg, false, G, &G2);	/* true if input is a read-only array */
-			gmt_grd_pad_off (GMT, G2);	/* Undo pad if one existed */
-			gmt_sort_array (GMT, G2->data, G2->header->nm, GMT_FLOAT);
-			median = (n%2) ? G2->data[n/2] : 0.5*(G2->data[n/2-1] + G2->data[n/2]);
-			for (ij = 0; ij < n; ij++) G2->data[ij] = (float)fabs (G2->data[ij] - median);
-			gmt_sort_array (GMT, G2->data, n, GMT_FLOAT);
-			scale = (n%2) ? 1.4826 * G2->data[n/2] : 0.7413 * (G2->data[n/2-1] + G2->data[n/2]);
-			if (new_grid) {	/* Free the temporary grid */
-				if (GMT_Destroy_Data (API, &G2) != GMT_NOERROR) {
-					GMT_Report (API, GMT_MSG_NORMAL, "Failed to free G2\n");
-				}
-			}
+		if (Ctrl->L.norm && gmt_M_is_geographic (GMT, GMT_IN)) {	/* Must use spherical weights */
+			W = gmt_duplicate_grid (GMT, G, GMT_DUPLICATE_ALLOC);
+			gmt_get_cellarea (GMT, W);
+		}
+		
+		if (Ctrl->L.norm & 1) {	/* Calculate the median and MAD */
+			z_median = gmt_grd_median (GMT, G, W, false);
+			z_scale = gmt_grd_mad (GMT, G, W, &z_median, false);
 		}
 		if (Ctrl->L.norm & 2) {	/* Calculate the mean, standard deviation, and rms */
-			x = (double)n;
-			stdev = (n > 1) ? sqrt (sum2 / (x-1)) : GMT->session.d_NaN;
-			rms = (n > 0) ? sqrt (sum2 / x + mean * mean) : GMT->session.d_NaN;
-			mean = (n > 0) ? mean : GMT->session.d_NaN;
+			z_mean = gmt_grd_mean (GMT, G, W);	/* Compute the [weighted] mean */
+			z_stdev = gmt_grd_std (GMT, G, W);	/* Compute the [weighted] stdev */
+			z_rms = gmt_grd_rms (GMT, G, W);		/* Compute the [weighted] rms */
 		}
+		if (Ctrl->L.norm & 4) {	/* Calculate the mode and lmsscale */
+			z_mode = gmt_grd_mode (GMT, G, W, false);
+			z_lmsscl = gmt_grd_lmsscl (GMT, G, W, &z_mode, false);
+		}
+		if (W) gmt_free_grid (GMT, &W, true);
 
 		if (gmt_M_is_geographic (GMT, GMT_IN)) {
 			if (gmt_M_grd_is_global(GMT, G->header) || (G->header->wesn[XLO] < 0.0 && G->header->wesn[XHI] <= 0.0))
@@ -414,19 +528,35 @@ int GMT_grdinfo (void *V_API, int mode, void *args) {
 
 		/* OK, time to report results */
 
-		if (Ctrl->I.active && Ctrl->I.status == GRDINFO_GIVE_REG_ORIG) {
+		i_status = Ctrl->I.status;
+		if (Ctrl->I.active && Ctrl->I.status == GRDINFO_GIVE_REG_IMG) {
+			if (!(strstr (G->header->command, "img2grd") || strstr (G->header->command, "img2mercgrd") || strstr (G->header->remark, "img2mercgrd"))) {
+				GMT_Report (API, GMT_MSG_NORMAL, "Could not find a -Rw/e/s/n string produced by img tools - returning regular grid -R\n");
+				i_status = GRDINFO_GIVE_REG_ORIG;
+			}
+		}
+
+		if (Ctrl->I.active && i_status == GRDINFO_GIVE_REG_ORIG) {
 			sprintf (record, "-R");
 			gmt_ascii_format_col (GMT, text, G->header->wesn[XLO], GMT_OUT, GMT_X);	strcat (record, text);	strcat (record, "/");
 			gmt_ascii_format_col (GMT, text, G->header->wesn[XHI], GMT_OUT, GMT_X);	strcat (record, text);	strcat (record, "/");
 			gmt_ascii_format_col (GMT, text, G->header->wesn[YLO], GMT_OUT, GMT_Y);	strcat (record, text);	strcat (record, "/");
 			gmt_ascii_format_col (GMT, text, G->header->wesn[YHI], GMT_OUT, GMT_Y);	strcat (record, text);
 			GMT_Put_Record (API, GMT_WRITE_TEXT, record);
-		} else if (Ctrl->I.active && Ctrl->I.status == GRDINFO_GIVE_INCREMENTS) {
+		}
+		else if (Ctrl->I.active && i_status == GRDINFO_GIVE_REG_IMG) {
+			char *c = strrchr (G->header->remark, 'R');
+			sprintf (record, "-%s", c);
+			GMT_Put_Record (API, GMT_WRITE_TEXT, record);
+		} else if (Ctrl->I.active && i_status == GRDINFO_GIVE_INCREMENTS) {
 			sprintf (record, "-I");
 			gmt_ascii_format_col (GMT, text, G->header->inc[GMT_X], GMT_OUT, GMT_Z);	strcat (record, text);	strcat (record, "/");
 			gmt_ascii_format_col (GMT, text, G->header->inc[GMT_Y], GMT_OUT, GMT_Z);	strcat (record, text);
 			GMT_Put_Record (API, GMT_WRITE_TEXT, record);
-		} else if (Ctrl->I.active && Ctrl->I.status == GRDINFO_GIVE_BOUNDBOX) {
+		} else if (Ctrl->I.active && i_status == GRDINFO_GIVE_BOUNDBOX) {
+			if (GMT_Set_Geometry (API, GMT_OUT, GMT_IS_POLY) != GMT_NOERROR) {	/* Sets output geometry */
+				Return (API->error);
+			}
 			if (GMT_Set_Geometry (API, GMT_OUT, GMT_IS_POLY) != GMT_NOERROR) {	/* Sets output geometry */
 				Return (API->error);
 			}
@@ -453,7 +583,7 @@ int GMT_grdinfo (void *V_API, int mode, void *args) {
 			gmt_ascii_format_col (GMT, text,   G->header->wesn[YLO], GMT_OUT, GMT_Y);	strcat (record, text);
 			GMT_Put_Record (API, GMT_WRITE_TEXT, record);
 		} else if (Ctrl->C.active && !Ctrl->I.active) {
-			if (API->mode) {	/* External interface, return as data with no leading text */
+			if (API->external) {	/* External interface, return as data with no leading text */
 				/* w e s n z0 z1 dx dy n_columns n_rows [x0 y0 x1 y1] [med scale] [mean std rms] [n_nan] */
 				gmt_M_memcpy (out, G->header->wesn, 4, double);	/* Place the w/e/s/n limits */
 				out[ZLO]   = G->header->z_min;		out[ZHI]   = G->header->z_max;
@@ -465,14 +595,17 @@ int GMT_grdinfo (void *V_API, int mode, void *args) {
 					out[col++] = x_max;	out[col++] = y_max;
 				}
 				if (Ctrl->L.norm & 1) {
-					out[col++] = median;	out[col++] = scale;
+					out[col++] = z_median;	out[col++] = z_scale;
 				}
 				if (Ctrl->L.norm & 2) {
-					out[col++] = mean;	out[col++] = stdev;	out[col++] = rms;
+					out[col++] = z_mean;	out[col++] = z_stdev;	out[col++] = z_rms;
 				}
 				if (Ctrl->M.active) {
 					out[col++] = (double)n_nan;
 				}
+				if (Ctrl->L.norm & 4) {
+					out[col++] = z_mode;	out[col++] = z_lmsscl;
+				}
 				GMT_Put_Record (API, GMT_WRITE_DATA, out);
 			}
 			else {	/* Command-line usage */
@@ -499,15 +632,19 @@ int GMT_grdinfo (void *V_API, int mode, void *args) {
 					strcat (record, sep);	gmt_ascii_format_col (GMT, text, y_max, GMT_OUT, GMT_Y);	strcat (record, text);
 				}
 				if (Ctrl->L.norm & 1) {
-					strcat (record, sep);	gmt_ascii_format_col (GMT, text, median, GMT_OUT, GMT_Z);	strcat (record, text);
-					strcat (record, sep);	gmt_ascii_format_col (GMT, text,  scale, GMT_OUT, GMT_Z);	strcat (record, text);
+					strcat (record, sep);	gmt_ascii_format_col (GMT, text, z_median, GMT_OUT, GMT_Z);	strcat (record, text);
+					strcat (record, sep);	gmt_ascii_format_col (GMT, text,  z_scale, GMT_OUT, GMT_Z);	strcat (record, text);
 				}
 				if (Ctrl->L.norm & 2) {
-					strcat (record, sep);	gmt_ascii_format_col (GMT, text,  mean, GMT_OUT, GMT_Z);	strcat (record, text);
-					strcat (record, sep);	gmt_ascii_format_col (GMT, text, stdev, GMT_OUT, GMT_Z);	strcat (record, text);
-					strcat (record, sep);	gmt_ascii_format_col (GMT, text,   rms, GMT_OUT, GMT_Z);	strcat (record, text);
+					strcat (record, sep);	gmt_ascii_format_col (GMT, text, z_mean, GMT_OUT, GMT_Z);	strcat (record, text);
+					strcat (record, sep);	gmt_ascii_format_col (GMT, text, z_stdev, GMT_OUT, GMT_Z);	strcat (record, text);
+					strcat (record, sep);	gmt_ascii_format_col (GMT, text,   z_rms, GMT_OUT, GMT_Z);	strcat (record, text);
 				}
 				if (Ctrl->M.active) { sprintf (text, "%s%" PRIu64, sep, n_nan);	strcat (record, text); }
+				if (Ctrl->L.norm & 4) {
+					strcat (record, sep);	gmt_ascii_format_col (GMT, text, z_mode,   GMT_OUT, GMT_Z);	strcat (record, text);
+					strcat (record, sep);	gmt_ascii_format_col (GMT, text, z_lmsscl, GMT_OUT, GMT_Z);	strcat (record, text);
+				}
 				GMT_Put_Record (API, GMT_WRITE_TEXT, record);
 			}
 		}
@@ -626,18 +763,25 @@ int GMT_grdinfo (void *V_API, int mode, void *args) {
 			}
 			if (Ctrl->L.norm & 1) {
 				sprintf (record, "%s: median: ", G->header->name);
-				gmt_ascii_format_col (GMT, text, median, GMT_OUT, GMT_Z);	strcat (record, text);
+				gmt_ascii_format_col (GMT, text, z_median, GMT_OUT, GMT_Z);	strcat (record, text);
 				strcat (record, " scale: ");
-				gmt_ascii_format_col (GMT, text, scale, GMT_OUT, GMT_Z);	strcat (record, text);
+				gmt_ascii_format_col (GMT, text, z_scale, GMT_OUT, GMT_Z);	strcat (record, text);
 				GMT_Put_Record (API, GMT_WRITE_TEXT, record);
 			}
 			if (Ctrl->L.norm & 2) {
 				sprintf (record, "%s: mean: ", G->header->name);
-				gmt_ascii_format_col (GMT, text, mean, GMT_OUT, GMT_Z);	strcat (record, text);
+				gmt_ascii_format_col (GMT, text,  z_mean, GMT_OUT, GMT_Z);	strcat (record, text);
 				strcat (record, " stdev: ");
-				gmt_ascii_format_col (GMT, text, stdev, GMT_OUT, GMT_Z);	strcat (record, text);
+				gmt_ascii_format_col (GMT, text, z_stdev, GMT_OUT, GMT_Z);	strcat (record, text);
 				strcat (record, " rms: ");
-				gmt_ascii_format_col (GMT, text, rms, GMT_OUT, GMT_Z);	strcat (record, text);
+				gmt_ascii_format_col (GMT, text,   z_rms, GMT_OUT, GMT_Z);	strcat (record, text);
+				GMT_Put_Record (API, GMT_WRITE_TEXT, record);
+			}
+			if (Ctrl->L.norm & 4) {
+				sprintf (record, "%s: mode: ", G->header->name);
+				gmt_ascii_format_col (GMT, text,   z_mode, GMT_OUT, GMT_Z);	strcat (record, text);
+				strcat (record, " lmsscale: ");
+				gmt_ascii_format_col (GMT, text, z_lmsscl, GMT_OUT, GMT_Z);	strcat (record, text);
 				GMT_Put_Record (API, GMT_WRITE_TEXT, record);
 			}
 			if (strspn(GMT->session.grdformat[G->header->type], "nc") != 0) {
@@ -662,7 +806,7 @@ int GMT_grdinfo (void *V_API, int mode, void *args) {
 			if (G->header->wesn[YLO] < global_ymin) global_ymin = G->header->wesn[YLO];
 			if (G->header->wesn[YHI] > global_ymax) global_ymax = G->header->wesn[YHI];
 		}
-		if ((Ctrl->T.mode & 2) == 0 && GMT_Destroy_Data (API, &G) != GMT_NOERROR) {
+		if (!delay && GMT_Destroy_Data (API, &G) != GMT_NOERROR) {
 			Return (API->error);
 		}
 	}
@@ -675,7 +819,7 @@ int GMT_grdinfo (void *V_API, int mode, void *args) {
 		global_xmax = ceil  (global_xmax / Ctrl->I.inc[GMT_X]) * Ctrl->I.inc[GMT_X];
 		global_ymin = floor (global_ymin / Ctrl->I.inc[GMT_Y]) * Ctrl->I.inc[GMT_Y];
 		global_ymax = ceil  (global_ymax / Ctrl->I.inc[GMT_Y]) * Ctrl->I.inc[GMT_Y];
-		if (gmt_M_is_geographic (GMT, GMT_IN)) {	/* Must make sure we don't get outside valid bounds */
+		if (!Ctrl->D.active && gmt_M_is_geographic (GMT, GMT_IN)) {	/* Must make sure we don't get outside valid bounds */
 			if (global_ymin < -90.0) {
 				global_ymin = -90.0;
 				GMT_Report (API, GMT_MSG_VERBOSE, "Warning: Using -I caused south to become < -90.  Reset to -90.\n");
@@ -690,7 +834,9 @@ int GMT_grdinfo (void *V_API, int mode, void *args) {
 				global_xmax = (global_xmin < 0.0) ? +180.0 : 360.0;
 			}
 		}
-		if (API->mode) {	/* External interface, return as data with no leading text */
+		if (Ctrl->D.active)
+			report_tiles (GMT, G, global_xmin, global_xmax, global_ymin, global_ymax, Ctrl);
+		else if (API->external) {	/* External interface, return as data with no leading text */
 			/* w e s n z0 z1 */
 			out[XLO] = global_xmin;		out[XHI] = global_xmax;
 			out[YLO] = global_ymin;		out[YHI] = global_ymax;
@@ -722,6 +868,7 @@ int GMT_grdinfo (void *V_API, int mode, void *args) {
 			global_zmin = gmt_quantile_f (GMT, tmp_grid, 0.5 * Ctrl->T.alpha, G->header->size);			/* "Left" quantile */
 			global_zmax = gmt_quantile_f (GMT, tmp_grid, 100.0-0.5* Ctrl->T.alpha, G->header->size);	/* "Right" quantile */
 			if (GMT_Destroy_Data (API, &G) != GMT_NOERROR) {	/* Delayed destroy due to alpha trimming */
+				gmt_M_free (GMT, tmp_grid);
 				Return (API->error);
 			}
 			if (gmt_M_file_is_memory (file_ptr))	/* Now free temp grid */
@@ -755,7 +902,7 @@ int GMT_grdinfo (void *V_API, int mode, void *args) {
 		global_xmax = ceil  (global_xmax / Ctrl->I.inc[GMT_X]) * Ctrl->I.inc[GMT_X];
 		global_ymin = floor (global_ymin / Ctrl->I.inc[GMT_Y]) * Ctrl->I.inc[GMT_Y];
 		global_ymax = ceil  (global_ymax / Ctrl->I.inc[GMT_Y]) * Ctrl->I.inc[GMT_Y];
-		if (gmt_M_is_geographic (GMT, GMT_IN)) {	/* Must make sure we don't get outside valid bounds */
+		if (!Ctrl->D.active && gmt_M_is_geographic (GMT, GMT_IN)) {	/* Must make sure we don't get outside valid bounds */
 			if (global_ymin < -90.0) {
 				global_ymin = -90.0;
 				GMT_Report (API, GMT_MSG_VERBOSE, "Warning: Using -I caused south to become < -90.  Reset to -90.\n");
@@ -770,12 +917,20 @@ int GMT_grdinfo (void *V_API, int mode, void *args) {
 				global_xmax = (global_xmin < 0.0) ? +180.0 : 360.0;
 			}
 		}
-		sprintf (record, "-R");
-		gmt_ascii_format_col (GMT, text, global_xmin, GMT_OUT, GMT_X);	strcat (record, text);	strcat (record, "/");
-		gmt_ascii_format_col (GMT, text, global_xmax, GMT_OUT, GMT_X);	strcat (record, text);	strcat (record, "/");
-		gmt_ascii_format_col (GMT, text, global_ymin, GMT_OUT, GMT_Y);	strcat (record, text);	strcat (record, "/");
-		gmt_ascii_format_col (GMT, text, global_ymax, GMT_OUT, GMT_Y);	strcat (record, text);
-		GMT_Put_Record (API, GMT_WRITE_TEXT, record);
+		if (Ctrl->D.active)
+			report_tiles (GMT, G, global_xmin, global_xmax, global_ymin, global_ymax, Ctrl);
+		else {
+			sprintf (record, "-R");
+			gmt_ascii_format_col (GMT, text, global_xmin, GMT_OUT, GMT_X);	strcat (record, text);	strcat (record, "/");
+			gmt_ascii_format_col (GMT, text, global_xmax, GMT_OUT, GMT_X);	strcat (record, text);	strcat (record, "/");
+			gmt_ascii_format_col (GMT, text, global_ymin, GMT_OUT, GMT_Y);	strcat (record, text);	strcat (record, "/");
+			gmt_ascii_format_col (GMT, text, global_ymax, GMT_OUT, GMT_Y);	strcat (record, text);
+			GMT_Put_Record (API, GMT_WRITE_TEXT, record);
+		}
+	}
+
+	if (delay && GMT_Destroy_Data (API, &G) != GMT_NOERROR) {	/* Delayed destroy due to -D+n */
+		Return (API->error);
 	}
 
 	if (!Ctrl->C.active && !Ctrl->T.active && projStr) {		/* Print the referencing info */
